﻿using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode
{
    [StorableClass]
    public class OnHitRobot : CodeNode
    {
        public override int MinimumArity { get { return 2; } }
        public override int MaximumArity { get { return 10; } }

        [Storable]
        public override string Prefix { get; set; }

        [Storable]
        public override string Suffix { get; set; }

        [StorableConstructor]
        private OnHitRobot(bool deserializing) : base(deserializing) { }
        private OnHitRobot(OnHitRobot original, Cloner cloner)
            : base(original, cloner)
        {
            this.Prefix = "\r\npublic void onHitRobot(HitRobotEvent e) {\r\n";
            this.Suffix = "\r\nexecute();\r\n}\r\n";
        }

        public OnHitRobot()
            : base("OnHitRobot", "This method is called when your robot collides with another robot.")
        {
            this.Prefix = "\r\npublic void onHitRobot(HitRobotEvent e) {\r\n";
            this.Suffix = "\r\nexecute();\r\n}\r\n";
        }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new OnHitRobot(this, cloner);
        }

        public override string Interpret(ISymbolicExpressionTreeNode node, System.Collections.Generic.IEnumerable<ISymbolicExpressionTreeNode> children)
        {
            string result = "";
            foreach (ISymbolicExpressionTreeNode c in children)
                result += "\r\n" + ((CodeNode)c.Symbol).Interpret(c, c.Subtrees);
            return this.Prefix + "\r\n" + result + "\r\n" + this.Suffix;
        }
    }
}