﻿using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode
{
    [StorableClass]
    public class NumericalExpression : CodeNode
    {
        public override int MinimumArity { get { return 1; } }
        public override int MaximumArity { get { return 1; } }

        [Storable]
        public override string Prefix { get; set; }

        [Storable]
        public override string Suffix { get; set; }

        [StorableConstructor]
        private NumericalExpression(bool deserializing) : base(deserializing) { }
        private NumericalExpression(NumericalExpression original, Cloner cloner)
            : base(original, cloner)
        {
            this.Prefix = "";
            this.Suffix = "";
        }

        public NumericalExpression()
            : base("NumericalExpression", "A NumericalExpression.")
        {
            this.Prefix = "";
            this.Suffix = "";
        }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new NumericalExpression(this, cloner);
        }

        public override string Interpret(ISymbolicExpressionTreeNode node, System.Collections.Generic.IEnumerable<ISymbolicExpressionTreeNode> children)
        {
            var enumerator = children.GetEnumerator();

            if (!enumerator.MoveNext()) throw new System.Exception("NumericalExpression was not given a child.");
            var symbol = enumerator.Current.Symbol;

            if (!(symbol is Number || symbol is NumericalOperation || symbol.GetType().GetInterface(typeof(INumericalMethod).FullName) != null))
                throw new System.Exception("NumericalExpression was given a child of type " + symbol.GetType().ToString() +
                    ". The expected child must be of type " + typeof(Number).ToString() + " or " +
                    typeof(NumericalOperation).ToString() + " or " +
                    typeof(INumericalMethod).ToString() + ".");

            string result = ((CodeNode)symbol).Interpret(enumerator.Current, enumerator.Current.Subtrees);
            if (enumerator.MoveNext()) throw new System.Exception("NumericalExpression was given more than one child.");

            return this.Prefix + result + this.Suffix;
        }
    }
}
