﻿using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode
{
    [StorableClass]
    public class Fire : CodeNode
    {
        public override int MinimumArity { get { return 1; } }
        public override int MaximumArity { get { return 1; } }

        [Storable]
        public override string Prefix { get; set; }

        [Storable]
        public override string Suffix { get; set; }

        [StorableConstructor]
        private Fire(bool deserializing) : base(deserializing) { }
        private Fire(Fire original, Cloner cloner)
            : base(original, cloner)
        {
            this.Prefix = "setFire(";
            this.Suffix = ");";
        }

        public Fire()
            : base("Fire", "Immediately fires a bullet.")
        {
            this.Prefix = "setFire(";
            this.Suffix = ")";
        }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new Fire(this, cloner);
        }

        public override string Interpret(ISymbolicExpressionTreeNode node, System.Collections.Generic.IEnumerable<ISymbolicExpressionTreeNode> children)
        {
            var enumerator = children.GetEnumerator();

            if (!enumerator.MoveNext()) throw new System.Exception("Fire was not given a child.");
            var symbol = enumerator.Current.Symbol;

            if (!(symbol is ShotPower))
                throw new System.Exception("Fire was given a child of type " + symbol.GetType().ToString() +
                    ". The expected child must be of type " + typeof(ShotPower).ToString() + ".");

            string result = ((CodeNode)symbol).Interpret(enumerator.Current, enumerator.Current.Subtrees);
            if (enumerator.MoveNext()) throw new System.Exception("Fire was given more than one child.");

            return this.Prefix + result + this.Suffix;
        }
    }
}